@@ -26,3 +26,15 @@ pub async fn get_files(
 
     Ok(files)
 }
+
+pub async fn get_file(
+    addon_id: usize,
+    file_id: usize,
+) -> Result<AddonFileDetail, Box<dyn std::error::Error>> {
+    let url = format!("{}/addon/{}/file/{}", BASE_URL, addon_id, file_id);
+    let body = reqwest::get(url).await?.text().await?;
+
+    let file: AddonFileDetail = serde_json::from_str(&body)?;
+
+    Ok(file)
+}
